;DDS Controller Program
; 16F630 version
;
; Important Note:
; To avoid Assembler errors, ensure that the 'Enable Case Sensitivity' option
; is un-checked in the MPASM tab of the configuration window in the MPLAB X
; software.
;
; 2020-09-28
; - Fixed a bug that prevented the AD9850 from working in sweep mode. This was
;   due to the same problem mentioned in the 2020-07-28 bug fix. The wrong
;   control bits were being sent to the DDS. The AD9851 was not affected by
;   this bug.
;
; 2020-08-15
; - Corrected a rounding error that occurred in the calculation of
;   the frequency tuning word (FTW). The error introduced by this bug would
;   have resulted in less than a fraction of 1 Hz frequency error. So it's
;   unlikely that it had any noticeable effect on operation.
;
; 2020-07-28
; - Fixed bug that sent wrong control bits to DDS when using AD9850
;
; 2017-12-16
; - Added code to insert commas (or other character) between groups of 3 digits
;   on LCD display. This uses conditional assembly directives. So there is
;   no option to turn it on or off by the end user.
;
; 2017-12-15
; - Added DDS reset code to work with RC network & clk signal
;
; 2017-12-10 Major revamp - version for non-lookup table calculation method
; - FTW calculation no longer uses lookup table. Now calculated from
;   base FTW coefficient in EEram
; - Added config mode. Holding down a key 0..3 at startup enters config mode 
; - Config mode allows selection of 125 or 180 MHz ref osc.
;   Conditional assembly no longer req'd for different ref frequency
; - Rearranged PB and Rot Sw input bits to simplify & speed up detection
;   of sweep mode exit condition
; - Added frequency markers in sweep mode. At fixed positions for now.
;
;
; **** End of Revision Notes ****
;
;
    list ; Turn on list mode
; Assembler directives
;    #define Clock20MHz  ;Comment out this line if using a 4 MHz clock frequency
    #define UseSeparators ;Comment out if no commas required between groups of 3 digits
;
; I/O Port Assignments
; 
; A0 - Output - LCDenable
; A1 - Output - DDSfq_ud
; A2 - Output -  DDSclk; KBD shift reg clk; D/A shift reg clk
; A3 - Input* - Serial in from Keypad/SelSw/Pushbutton Shift Register
; A4 - Clock - Xtal
; A5 - Clock - Xtal
; C0 - Output - LCD D4; KBD row select 0; DDS serial data out
; C1 - Output - LCD D5; KBD row select 1; D/A serial data out
; C2 - Output - LCD D6; KBD row select 2
; C3 - Output - LCD D7; KBD Load shift reg
; C4 - Output - LCD register select
; C5 - Output - LCD read/write
;
;
;LDC display
    #define LCDport portC   ;Bits 0..3 used for data
    #define LCDrs portC,4
    #define LCDrw portC,5
    #define LCDenable portA,0 ; *** moved from portA,5
;DDS chip
    #define DDSfq_ud portA,1 ;***
    #define DDSclk portA,2 ;***
    #define DDSdata portC,0
;D/A output
    #define DAdata portC,1
;Input
    #define KeyIn   portA,3  ; input only 
    #define KeyClk  portA,2  ;was A.5, reassigned to allow external clock
    #define KeyLoad  portC,3
    #define KeyRow2  portC,2
    #define KeyRow1  portC,1
    #define KeyRow0  portC,0

;   Other Misc. Equates
eebase equ 0x2100 ;base address of EERAM
SeparatorChar equ ","
#ifdef UseSeparators
;Position of Mode character varies depending on whether separators are used.
ModeCharPosn equ 0x01
#else
ModeCharPosn equ 0x04
#endif


#ifdef Clock20MHz
; Delay constants for 20 MHz clock
; 20 MHz constants are approximate, not fine tuned.
LongDelayMpy equ d'75'
DebounceConst equ d'120'
Delay10msLong equ d'60' ;Delay constants for 4.000 MHz clock
Delay10msShort equ d'248'
Delay1msLong equ d'5'
Delay1msShort equ d'72'
Delay40usShort equ d'65'
#else
; Delay constants for 4 MHz clock
; 1ms and 10ms are exact, for critical timing
LongDelayMpy equ d'15'
DebounceConst equ d'24'
Delay10msLong equ d'12' ;Delay constants for 4.000 MHz clock
Delay10msShort equ d'248'
Delay1msLong equ d'1'
Delay1msShort equ d'72'
Delay40usShort equ d'12'
#endif
; The following is relative to program code not absolute time. Therefore
; the value will be the same regardless of processor clock frequency
MarkerTime equ d'4' ; sweep marker delay

    #include p16f630.inc

#ifdef Clock20MHz
;2017-03-04 added to use external crystal osc.
    __config _HS_OSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF
#else
;Otherwise use built in 4 MHz oscillator
    __config _INTRC_OSC_NOCLKOUT & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF
#endif



;Macro definitions
bank0 macro
	bcf STATUS,RP0
	endm
bank1 macro
	bsf STATUS,RP0
	endm
;More intuitive skip definitions
skpeq macro
	btfss status,z
	endm
skpneq macro
	btfsc status,z
	endm
skpneg macro
    btfsc status,c
    endm
skppos macro
    btfss status,c
    endm
dde macro arg ;Store a 16 bit value in adjacent memory locs
    de low arg
    de high arg
    endm
SaveAccHi macro rName ;Store 4 high bytes of acc to spec'd register set
    movlw rName
    call SaveAccHiInd
    endm
SaveAccLo macro rName ;Store 4 low bytes of acc to spec'd register set
    movlw rName
    call SaveAccLoInd
    endm
LoadAccHi macro rName ;Load 4 high bytes of acc from spec'd register set
    movlw rName
    call LoadAccHiInd
    endm
LoadAccLo macro rName ;Load 4 low bytes of acc from spec'd register set
    movlw rName
    call LoadAccLoInd
    endm
LoadRLo macro rName ;Load 4 low bytes of aux reg from spec'd register set
    movlw rName
    call LoadRxLoInd
    endm
LoadRHi macro rName ;Load 4 high bytes of aux reg from spec'd register set
    movlw rName
    call LoadRxHiInd
    endm
ClearNregisters macro nReg, rName ;Clear nReg registers starting at rName
    movlw nReg
    movwf xfrCount
    movlw rName
    call ClearNregistersInd
    endm
ShiftUpNregisters macro nReg, rName ;Shift nReg registers starting at rName
    movlw nReg
    movwf xfrCount
    movlw rName
    call ShiftUpNregistersInd
    endm
ReadNeeram macro nReg, rName, eAdr
    movlw nReg
    movwf xfrCount
    movlw eAdr
    movwf temp
    movlw nName
    call EEramToRegInd
    endm
;
;variable memory area
    cblock 0x20
;    wsave,statsave,LastEnc,saveIntcon ;Misc. save registers
    dlyctrL,dlyctrH ;for delay routines
;   LCD display variables:
    LCDdatatemp
    LCDmode ;command or data in bit 4
    chrOffset ; offset into ascii table
;   DDS variables:
    freqDigits:8 ;8 digit frequency in Hz, F(0) is LSB, F(7) is MSB
    offsetDigits:8 ;8 digit sweep offset in Hz Offset(0) is LSB etc.
;    DDSctlBits ;Control bits for DDS
    DDSmode ;flag bits for DDS mode; b0=centre/offset freq enter b1=on/off sweep
    DDSswpIndex,DDSswpIncr ; current sweep freq iteration 0..255
    cw0,cw1,cw2,cw3 ;Storage for DDS control word
    sw0,sw1,sw2,sw3 ;Storage for sweep offset word
    sf0,sf1,sf2,sf3 ;Storage for sweep start freq word
    fqi_f,fqi0,fqi1,fqi2,fqi3 ;instantaneous frequency ctl word (FTW)
;   Math Library variables
    acc0,acc1,acc2,acc3,acc4,acc5,acc6,acc7 ;accumulator for 32/64 bit math
    R0,R1,R2,R3,R4,R5,R6,R7 ; 32/64 bit math aux. registers
    count,count1,temp ;misc temporary registers
    xfrCount ;used to count registers moved by the load, save & clear routines
    lastKey ; misc. keyboard variables
    ctrlWord ;0x00 for AD9850, 0x01 for AD9851
    reg64 ;last two available unused registers
    endc

;DDSmode register bits

    #define KBlatch DDSmode,7
    #define KBinMode DDSmode,6
    #define DDSLastMode1 DDSmode,5
    #define DDSLastMode0 DDSmode,4
    #define KBinModeMask b'01000000'
    #define SelSwMask b'00000011'
    #define DDSlastModeMask b'00110000'


;Reset and interrupt vectors
RST code 0x0
    goto	Main	;Entry point for program from Reset
    nop
    nop
    nop
    return	;interrupt vector (no interrupts in this project)
;Lookup tables
ModeSymbol
    andlw 0x03
    addwf pcl,f
ModeSymbol_0
    retlw 0x7E ; right arrow code (for mode 0)
    retlw "+" ; mode 1 - increment/decrement
    retlw "s" ; 255 step sweep mode
    retlw "S" ; 128 step sweep mode
ExecCmd
    ; Jump table for Selector Switch and pushbutton input commands

    ; Goto's are used here; no need for superfluous call nesting
    ; Unimplemented commands return directly to caller
    ; bits 2..3 are rotary mode switch input
    ; bits 0 and 1 are frequency increment and decrement PB's
    andlw 0x0F
    addwf pcl,f

    goto KBDnewKey ;cmd 0
    goto PBreset ;reset PB latch (detects release of pushbutton)
    goto Sweep255 ;
    goto Sweep128 ;cmd 3

    goto PBSweepFrqDec  ;cmd 4 (optional decrement command in kbd mode) 
    goto PBSweepFrqDec  ;cmd 5
    goto Sweep255 ;cmd 6
    goto Sweep128 ;cmd 7 

    goto PBSweepFrqInc  ;cmd 8 (optional increment command in kbd mode) 
    goto PBSweepFrqInc ;cmd 9
    goto Sweep255 ;cmd 10
    goto Sweep128 ;cmd 11

    return  ;cmd 12 (invalid state corresponding to both buttons pressed)
    return  ;cmd 13 (invalid state corresponding to both buttons pressed)
    goto Sweep255 ;cmd 15
    goto Sweep128 ;cmd 15 


;*****************************************************************************
;*****************************************************************************
;*****************************************************************************
;*********                                                        ************
;*********                                                        ************
;*********                 Start of Main Program                  ************
;*********                                                        ************
;*********                                                        ************
;*****************************************************************************
;*****************************************************************************
;*****************************************************************************
;
Main ;Start of main program
; Hardware initialization
    clrf status ; clear status reg
    clrf pclath ; clear PCH
    clrf intcon
    bank0 ; Sel Bank 0
    clrf portC
    clrf portA
    movlw 0x07  ;comparator off
    movwf cmcon ;disable comparator, enable digital portA
    bank1 ; Sel Bank 1
    call 0x3FF ; calls RETLW with factory setting
    movwf OSCCAL ; Set int OSC to factory calibrated
    clrf option_reg
    movlw b'00000000' ; Config port C for all outputs
    movwf TRISC
    movlw b'00001000' ; Config port A for all outputs except A3
    movwf TRISA
    movlw b'00000000'
    movwf wpua  ;set portA inputs weak pull-ups
; Interrupt initialization - No interrupts are used in this program.
    bank1
    clrf pie1   ;Disable all peripheral interrupts
    movlw b'00001000'
;    movwf ioca      ;interrupt on change for PortA input 3
;    movlw b'10001000'  ;Enable interrupts
    movlw b'00001000'  ;Disable interrupts
    movwf intcon    ;
; Program initialization
    bank0
    ;Clear keyboard registers
    clrf lastKey
    bcf KeyClk
    ; Set all DDS outputs low
    bcf DDSfq_ud
    bcf DDSclk
    clrf DDSmode ; set DDSmode to centre frequency input
    call InitDDS
; Init LCD
; Begin the LCD initialisation
; Power up timer should give delay, but do it anyway
    call Delay10mS ;LCD needs a 20ms delay after power up to allow it
    call Delay10mS ;to complete an internal initialization sequence.
    call LCDinit
    call LCDclear
    call Delay10mS
;
; Check for config mode by reading keyboard key press
    call configMode
; Load the saved control word value appropriate for the selected DDS IC
    movlw eeCtlWd - eeBase
    call getEEram
    movwf ctrlWord
; Display text labels on LCD
    call LCDhome
;SendStaticTxt
    movlw Msg01 - eeBase
    call LCDeeTxt
    call LCDline2
    movlw Msg02 - eeBase
    call LCDeeTxt

; DDS initialization
    bcf DDSfq_ud
    bsf DDSclk ; holding clock high for at least 1ms resets DDS
    call Delay10mS
    bcf DDSclk
    call Delay1mS
    call SetDDSserial ;DDS starts up in parallel mode. This sets serial mode
    call SweepZero
    call FreqZero
    call ModeDisplay ; display mode symbol
MainLoop
    call PBread ;read the pushbutton & mode sw. inputs
    ;see if input has changed
    swapf DDSmode,w
    xorwf DDSmode,w
    andlw DDSlastModeMask ;mask off everything except the 2 mode select bits
    skpnz
    goto DoCmd ;Mode hasn't changed, so skip mode display update
    call ModeDisplay ;update the mode character in the LCD display
    ;save new mode in lastmode bits
    movlw 0xFF
    xorlw DDSlastModeMask
    andwf DDSmode,f
    swapf DDSmode,w
    andlw DDSlastModeMask
    iorwf DDSmode,f
DoCmd 
    movf DDSmode,w ;Load the current mode & pushbutton inputs
    call ExecCmd ;and call jump table to handle inputs
    goto MainLoop ;repeat forever.
;  Project Subroutines
;
;
;  Keyboard routines
KBDnewKey ;See if new key pressed
    call PBreset
    movlw DebounceConst ; This is a simple debounce delay
    call Delay
    call KBDread ;returns zero if key up, else returns non zero keycode
    xorwf lastKey,f ;swap lastKey and w (new key)
    xorwf lastKey,w
    xorwf lastKey,f
    xorwf lastKey,w ;test if different
    skpnz ;has input changed?
    return ;no, so exit
    rlf lastKey,w ;test bit 7 for keydown
    btfss status,c
    return ;keyup, so return
    ;new key down, so update the frequency
    movlw b'10001011' ;"Clear" key code
    xorwf lastKey,w
    skpnz ;
    goto DoClear
    movlw b'10001010' ; Line switch key code
    xorwf lastKey,w
    skpnz ;
    goto SwitchLCDline ; handle Line switch key
;   Else Digit key has been pressed
;   Update either centre frequency or sweep offset depending on mode
    clrf DDSswpIndex	;clear the sweep index whenever a digit key is pressed
    btfsc KBinMode ;
    goto SweepInputKey
FreqInputKey
    ShiftUpNregisters 7,freqDigits+7 ;Shift each existing digit to the left
    movf lastKey,w
    andlw 0x0F
    movwf freqDigits	;then insert new digit at end
    goto FreqUpdate

FreqZero    ; Set DDS frequency to 0 Hz
    ClearNregisters 8,freqDigits
FreqUpdate  ;Calc ctl word from digit array and update DDS frequency
    call MakeCtlWd
    SaveAccLo cw0 ;move the acc to the ctlwd storage register
    call DisplayFreq
    LoadAccLo cw0
    call SendCtlWd
    return
SweepInputKey
    ShiftUpNregisters 7,offsetDigits+7 ;Shift each existing digit to the left
    movf lastKey,w
    andlw 0x0F
    movwf offsetDigits	;then insert new digit at end
    call DisplaySweep
    return
SweepZero    ; Set DDS sweep offset to 0 Hz
    ClearNregisters 8,offsetDigits
    call DisplaySweep
    return
SweepUpdate
    call MakeSwpWd ;generate the control word for the sweep
    SaveAccLo sw0 ;save it
    ;calculate the starting sweep freq
    call neg32
    LoadRLo cw0 ;load ctrl word into aux register
    call add32	;acc now contains start freq
    SaveAccLo sf0 ;save to the sweep freq word
    ; 2017-02-27 modified for 128 frequency increments instead of 255
    ;multiply sweep offset by 4 (was 2) so it extends on both sides of centre freq
    bcf status,c
    rlf sw0,f
    rlf sw1,f
    rlf sw2,f
    rlf sw3,f
; if sweep increments are 255 then skip the second 2x
    movf DDSswpIncr,f
    skpnz
    return
    bcf status,c
    rlf sw0,f
    rlf sw1,f
    rlf sw2,f
    rlf sw3,f ;if there's a carry here, then the offset is too big
    ;Could add code to check carry and disable sweep if carry set,
    ; but currently does no harm except sweep display will show a discontinuity
    ; where overflow occurs.
    return
SwitchLCDline ;Toggle DDSmode frequency input mode 
    movlw KBinModeMask
    xorwf DDSmode,f


; 2017-12-16 Added option to use comma or dot separators between groups
; of 3 digits. This requires adjustment to the position of the mode indicator
; character which is now a variable: ModeCharPosn
ModeDisplay ; code to display mode on LCD.
    ;Clear the arrow from both LCD lines
    movlw ModeCharPosn 
    call LCDgoto
    movlw " "
    call send_data
    movlw ModeCharPosn+40 ; 2017-12-16 adjusted position to allow room for separators
    call LCDgoto
    movlw " "
    call send_data
    ;check mode to see if an arrow should be displayed
    movlw SelSwMask
    andwf DDSmode,w
    skpz
    goto DispModes123
    movlw ModeCharPosn
    btfsc KBinMode ;Select line 1 or 2 depending on mode
    movlw ModeCharPosn+40
    call LCDgoto
    call ModeSymbol_0 ; load char code for right arrow
    call send_data
    return
DispModes123 ; Display correct symbol for current mode
    movlw ModeCharPosn ; 2017-12-16 adjusted position to allow room for separators
    call LCDgoto
    movf DDSmode,w
    call ModeSymbol
    call send_data
    return

DoClear ;Handle clear pushbutton
    btfsc KBinMode ;check current mode
    goto SweepZero ;clear line 2
    goto FreqZero ;clear line 1

PBSweepFrqInc ; Called by pushbutton to Increment frequency
    btfsc KBlatch ;check if already done for this PB press
    return  ;Button is still down from previous execution, so ignore.
    bsf KBlatch ;Else, set the PB latch so this is done only once
    LoadAccLo freqDigits
    LoadAccHi freqDigits+4
    LoadRLo offsetDigits
    LoadRHi offsetDigits+4
    call AddDecimal
    SaveAccLo freqDigits
    SaveAccHi freqDigits+4
    goto FreqUpdate
PBSweepFrqDec ; Called by pushbutton to Decrement frequency
    btfsc KBlatch ;check if already done for this PB press
    return    ;Button is still down from previous execution, so ignore.
    bsf KBlatch ;Else, set the PB latch so this is done only once
    LoadAccLo offsetDigits
    LoadAccHi offsetDigits+4
    call NegDecimal ; negate the frequency offset
    LoadRLo freqDigits
    LoadRHi freqDigits+4
    call AddDecimal
    SaveAccLo freqDigits
    SaveAccHi freqDigits+4
    goto FreqUpdate
PBreset ; clear the PB latches
    bcf KBlatch
    return

SweepReset ;Reset the current frequency to the start frequency
    LoadAccLo sf0
    SaveAccLo fqi0
    clrf fqi_f
    clrf DDSswpIndex ;zero the offset index
    call SendCtlWd
    return

;The fast and slow sweep modes share most of the same code but with
; different entry points.
Sweep255
    movlw b'00001000'
    movwf count1 ;remember current sweep mode
    movlw 0x00
    movwf DDSswpIncr
    goto FSentry
Sweep128 ;Combine ctrl word increment and send to DDS for max speed
    ;Need to generate the sweep control word here
    clrf count1 ;remember current sweep mode
    movlw 0x80
    movwf DDSswpIncr
FSentry
    call SweepUpdate ;Calculate sweep control word
    bsf KeyLoad ;start with load bit high
    bcf KeyClk	;and clock low
    bcf DDSfq_ud
    bcf DDSclk
    call clrAcc64
    LoadRHi sw1 ;Sweep increment into aux register (offset by 1 byte)
    clrf r7
    movf sw0,w
    movwf r3
;2020-09-28 fixed bug where wrong control word sent to DDS chip (AD9850 only)
    movf ctrlWord,w
;    movlw 0x01 ;DDS ctrl bits: 6x freq mult, power up, and zero phase
    movwf acc0  ; out of order but doesn't matter 
LoopFSsweep
;   Tight execution loop starts here
    clrf acc3 ;acc3 holds fractional part for this loop
    movf sf0,w
    movwf acc4
    movf sf1,w
    movwf acc5
    movf sf2,w
    movwf acc6
    movf sf3,w
    movwf acc7
    bcf count1,0 ;clear the marker flag in case it's set
;Change the next line to use a variable increment
    movf DDSswpIncr,w ; reset increment counter
    movwf DDSswpIndex
    goto LoopFSfreqEntry
LoopFSfreq
;   Even tighter execution loop starts here
    call add40	;increment to the next frequency
    btfsc count1,0 ;see if marker delay flag is set
    call MarkerDelay ; put marker delay here to avoid end delays

LoopFSfreqEntry
    movlw 0x20	; init counter for sending 32 bits (32=0x20) to DDS
    movwf count
LoopFSbits1
    bcf DDSdata	;default clear data bit
    rrf acc4,w  ;get LSbit into carry
    rrf acc0,f  ;shift all 5 ctrl bytes right
    rrf acc7,f
    rrf acc6,f
    rrf acc5,f
    rrf acc4,f
    ;next bit to be sent is in carry
    btfsc status,c  ;if 1, then set data out to 1
    bsf DDSdata
    bsf DDSclk	;toggle the clock to send the bit
    bcf DDSclk
    decfsz count,f
    goto LoopFSbits1    ;repeat until all bits sent
    movlw 0x08	; init counter for sending 8 bits to DDS and D/A
    movwf count
LoopFSbits2 ;Second loop outputs last 8 bits of ctlword and D/A together
    bcf DDSdata	;default set DDS data bit
    bsf DAdata	;inverted for D/A
    rrf acc4,w  ;get LSbit into carry
    rrf acc0,f  ;shift all 5 ctrl bytes right
    rrf acc7,f
    rrf acc6,f
    rrf acc5,f
    rrf acc4,f
    ;next bit to be sent is in carry
    btfsc status,c  ;if 1, then set DDS data out to 1
    bsf DDSdata
    rrf DDSswpIndex,w
    rrf DDSswpIndex,f
    btfsc status,c  ;if 1, then set D/A data out to 0 (inverted output)
    bcf DAdata
    bsf DDSclk	;toggle the clock to send the bit
    bcf DDSclk
    decfsz count,f
    goto LoopFSbits2    ;repeat until all bits sent
    bsf DDSfq_ud    ;trigger frequency update and D/A latch
    bcf DDSfq_ud
;
;  Insert code here to check DDSswpIndex low 5 or 6 bits for zero and call delay
;   This puts intensified markers at 25%, 75% and centre frequency
    movlw 0x3F
    btfss count1,3
    movlw 0x1F
    andwf DDSswpIndex,w
    btfsc status,z
    bsf count1,0 ; set delay flag
;
;
    decfsz DDSswpIndex,f
    goto LoopFSfreq ;Do next frequency
    ;check if sweep mode input is still true.
    bcf KeyLoad ; then toggle load low then high
    bsf KeyLoad ; to load parallel - rot sw D1 now in portA.3
    movf count1,w
    xorwf portA,w
    addlw b'00001000'
    btfsc status,dc ;Run status is now at A.3
    goto LoopFSsweep
    ;Done here
    ;Reset frequency to centre frequency before exiting
    LoadAccLo cw0
    call SendCtlWd
;    call ModeDisplay ;update mode indication
    return
MarkerDelay
    movlw MarkerTime
    call delay
    bcf count1,0 ; clear the marker delay flag
    return
;
configMode
; Configure mode is entered by holding down a key (0..3) at startup
    clearNregisters 8,freqdigits
    bcf KeyRow2
    bcf KeyRow1
    bsf KeyRow0 ; set row 0
    call KBDinit
    bcf status,c
    rlf count,f ;double the normal 4 count to 8
configLoop
    swapf portA,w ;get serial input bit into w.7
    addlw 0x80 ;shift w.7 into carry
    rlf temp,f ;shift into temp
    bsf KeyClk	;toggle clock to get next bit
    bcf KeyClk
    decfsz count,f
    goto configLoop
    movlw 0x0F
    andwf temp,w
    btfsc status,z
    return ;no keypress so return
    ;At this point, w contains a number 1..15 based on combination
    ; of keys 0,1,2,3 held down on power up. So this can be used for other
    ; config functions as well
    ; for now the only options are:
    ; - 0 key: set 125 MHz ref
    ; - 1 key: set 180 MHz ref
    
    andlw 0x01 ;reduce options to the 0 key
    movlw ftwFactorySetting1-ftw
    btfss status,z
    movlw ftwFactorySetting2-ftw
    movwf temp ; offset between read and write addresses
    movlw 0x07 ; number of bytes to transfer
    movwf xfrCount ; current count in this reg
    movlw ftw-eebase
    bank1
    movwf eeadr ; write address
xferloop
    movf temp,w ;offset between read and write
    bank1
    addwf eeadr,f ;address of factory value
    bsf eecon1,rd ;read value from eeram
    movf temp,w
    subwf eeadr,f ;address to write
    call EEwrite
    incf eeadr,f
    decfsz xfrCount
    goto xferloop
    bank0
    call LCDhome
    bank1
    movf temp,w
    addwf eeadr,w ; message text immediately follows ftw base data
    bank0
    call LCDeeTxt ; display ref frequency confirmation message
    movwf 0xff
    call LongDelay
    call LCDclear
    call Delay10mS
    return

KBDread	;returns with w.7 set if key is down, and key code in bits 0..3 

    clrf count1 ;Keyboard matrix Row counter
    bcf KeyRow2
    bcf KeyRow1
    bsf KeyRow0 ; set row 0
    call KBDreadRow
    skpz ;non zero if returned with a keydown value
    return
    incf count1,f ; bump the row count
    bcf KeyRow0
    bsf KeyRow1 ; set row 1
    call KBDreadRow
    skpz ;non zero if returned with a keydown value
    return
    incf count1,f ; bump the row count
    bcf KeyRow1
    bsf KeyRow2 ; set row 2
    call KBDreadRow
    return
KBDreadRow
;Keyboard serial input routine
    call KBDinit

LoopSkip4 ;Skip first 4 bits
    bsf KeyClk	;toggle clock to get next bit
    bcf KeyClk
    decfsz count,f ;skip columns 0..3
    goto LoopSkip4
    call KBDcountInit ;reset count for rows 4..7

LoopKBDRR
    swapf portA,w ;read serial in bit at portA.3 to w.7
    addlw 0x80 ;shift w.7 to carry
    btfsc status,c 
    goto KeyFound
    bsf KeyClk	;toggle clock to get next bit
    bcf KeyClk
    decfsz count,f ;try next matrix column
    goto LoopKBDRR
    clrw    ;return zero to indicate no key down
    return
KeyFound
    decf count,f ;This is the active column no.
    ;combine column and row to get complete key code
    rlf count1,f ;mpy row no. by 4
    rlf count1,w
    andlw b'00001100'
    addwf count,w ;add in the column no. to get the 4 bit key code
    iorlw b'10000000' ;set bit 7 as keydown flag
    return
KBDinit	;set up i/o lines for Keyboard read 
    bsf KeyLoad ;start with load bit high
    bcf KeyClk	;and clock low
    bcf KeyLoad ; then toggle load low then high
    bsf KeyLoad ; to load parallel
KBDcountInit
    movlw 0x04	; read 4 inputs
    movwf count
    return
;Pushbutton read
PBread
    movlw DebounceConst
    call delay
    call KBDinit ;Init column counter
LoopPBR1 ;now read the push button row 0 columns 0..3 (includes BCD mode sw.)
    swapf portA,w ;read serial in bit at portA.3 to w.7
    addlw 0x80 ;shift w.7 to carry
    rrf temp,f	;shift data bit into temp
    bsf KeyClk	;toggle clock to get next bit
    bcf KeyClk
    decfsz count,f
    goto LoopPBR1 ; read next column
    movlw b'11110000' ;mask to clear PB state
    andwf DDSmode,f
    swapf temp,w
    andlw b'00001111' ;clear extraneous bits
    iorwf DDSmode,f ;store PB state in DDS mode low nibble
    return

;The following routine was used for initial testing of communications between
;the PIC controller and the DDS chip. It loads a hard coded 1 MHz FTW to the 
;accumulator registers for transmission to the DDS chip.
;No longer needed. So it's commented out to save program space
;Set1MHz ;For 180MHz ref clock only.
;    movlw 0x01
;    movwf acc3
;    movlw 0x6C
;    movwf acc2
;    movlw 0x16
;    movwf acc1
;    movlw 0xC1
;    movwf acc0
;    return
InitDDS 
    bcf DDSfq_ud
    bcf DDSclk
    return
SetDDSserial
;Need to toggle clk once to parallel load W0, thus setting serial mode
    bsf DDSclk
    bcf DDSclk
    bsf DDSfq_ud    ; toggle fq_ud to enable
    bcf DDSfq_ud
    return
SendCtlWd   ;Send Freq Ctrl Word to DDS chip
    movf ctrlWord,w
    movwf acc4  ; sets DDS to 6x freq mult (AD9851 only), power up, and zero phase
    bcf DDSfq_ud
    bcf DDSclk
    movlw 0x28	; send 40 bits (40=0x28)
    movwf count
LoopSCW
    bcf DDSdata	;default clear data bit
    rrf acc4,f  ;shift all 5 ctrl bytes right
    rrf acc3,f
    rrf acc2,f
    rrf acc1,f
    rrf acc0,f  ;next bit to be sent is in carry
    btfsc status,c  ;if 1, then set data out to 1
    bsf DDSdata
    bsf DDSclk	;toggle the clock to send the bit
    bcf DDSclk
    decfsz count,f
    goto LoopSCW    ;repeat until all bits sent
    bsf DDSfq_ud    ;trigger frequency update
    bcf DDSfq_ud
    return

#ifdef UseSeparators
; 2017-12-16 new version includes commas between groups of 3
DisplaySweep
    movlw 0x43	;line 2 position of sweep freq
    call LCDgoto
    movlw offsetDigits+7 ;load addr of MSD in offsetDigits digits array
    goto DispEntry
DisplayFreq ;Send the frequency to the LCD display
    movlw 0x03	; LCD cursor start position is 0
    call LCDgoto ; move the cursor to start posn
    movlw freqDigits+7	;load addr of MSD in frequency digits array
DispEntry
    movwf FSR	; store addr of MSD in indirect ptr reg
    movlw 0x07  ; load count of digits less one
    movwf count	; character count
    movlw 0x02 ; digit count before first separator char
    movwf count1
    call LCDinitZerBlank ; Blank leading zeroes
LoopDF
    movf indf,w	;load next digit value
    call DigitOut ;send it to LCD display
    decfsz count1,f
    goto ContDispEntry
    movlw ","
    btfss chrOffset,4
    movlw " " ; Still in suppressed leading zeros, so don't display comma
    call send_data
    movlw 0x03 ; reset group counter
    movwf count1
ContDispEntry
    decf FSR,f
    decfsz count,f
    goto LoopDF ; repeat until only one digit left
    movf indf,w
    call DigitOutLast ;do last digit
    movlw " "
    call send_data
    movlw "H"
    call send_data
    movlw "z"
    call send_data
    return
#else ;don't use separators

DisplaySweep
    movlw 0x45	;line 2 position of sweep freq
    call LCDgoto
    movlw offsetDigits+7 ;load addr of MSD in offsetDigits digits array
    goto DispEntry
DisplayFreq ;Send the frequency to the LCD display
    movlw 0x05	; LCD cursor start position is 0
    call LCDgoto ; move the cursor to start posn
    movlw freqDigits+7	;load addr of MSD in frequency digits array
DispEntry
    movwf FSR	; store addr of MSD in indirect ptr reg
    movlw 0x07  ; load count of digits less one
    movwf count	; character count
    call LCDinitZerBlank ; Blank leading zeroes
LoopDF
    movf indf,w	;load next digit value
    call DigitOut ;send it to LCD display
    decf FSR,f
    decfsz count,f
    goto LoopDF ; repeat until only one digit left
    movf indf,w
    call DigitOutLast ;do last digit
    movlw " "
    call send_data
    movlw "H"
    call send_data
    movlw "z"
    call send_data
    return
#endif ;use separators



;DisplayCtrWrd	;Send the control word to LCD display
;commented out because not currently used
;    movlw 0x45
;    call LCDgoto   ;display on right half of second line
;    movf acc3,w
;    call HexOut
;    movf acc2,w
;    call HexOut
;    movf acc1,w
;    call HexOut
;    movf acc0,w
;    call HexOut
;    return
MakeSwpWd   ;Generate DDS sweep control word
    call clrAcc64 ; Init the acc to zero
    movlw offsetDigits+7
    goto MakeWdEntry
MakeCtlWd   ;Generate DDS frequency control word
    call clrAcc64 ; Init the acc to zero
    movlw freqDigits+7	;load addr of frequency digits array
MakeWdEntry
    movwf FSR	; and store in indirect ptr reg
    movlw 0x08  ; load count of digits
    movwf count
;    call clrAcc64 ; Init the acc to zero
    goto LoopMCWenter ;skip the x10 for the 1st digit
LoopMCW
    call AccTimes10
LoopMCWenter
    movf indf,w
    call AddDigit
    decf fsr,f
    decfsz count,f
    goto LoopMCW
;
;All digits have been processed.
;
    btfsc acc3,7    ; Check fractional part to see if round up is req'd
    incfsz acc4,f   ;Yes, so increment integer part, and propagate carries.
    goto MCWshiftWord
    incfsz acc5,f
    goto MCWshiftWord
    incfsz acc6,f
    goto MCWshiftWord
    incf acc7,f
MCWshiftWord    ;move the result from acc4..acc7 to acc0..acc3
    LoadAccLo acc4
    return  ;Frequency/sweep Control Word is in acc0..acc3
;
AddDigit    ; Add next frequency coefficient to acc and mpy by 10
    ;coefficient is equal to digit value x fBase
    clrf r0 ;clear aux reg r
    clrf r1
    clrf r2
    clrf r3
    clrf r4
    clrf r5
    clrf r6
    clrf r7
    movwf temp ; save digit value
    movf temp,f ;test for zero
    btfsc status,z
    goto mpyDone
DigitMpyLoop ;repeated additions is simplest way to multiply by 0..9
    movlw ftw-eebase ;FTW base is in EERAM starting at this address
    call getEEram  
    bcf status,c
    addwf r0,f
    btfsc status,c
    call carryR64_1
    call getNxtEEram
    addwf r1,f
    btfsc status,c
    call carryR64_2
    call getNxtEEram
    addwf r2,f
    btfsc status,c
    call carryR64_3
    call getNxtEEram
    addwf r3,f
    btfsc status,c
    call carryR64_4
    call getNxtEEram
    addwf r4,f
    btfsc status,c
    call carryR64_5
    call getNxtEEram
    addwf r5,f
    btfsc status,c 
    call carryR64_6
    decfsz temp,f
    goto DigitMpyLoop

; 2020-08-15 Fixed rounding routine
    rlf r0,f ;move high bit of LSB to carry
    btfsc status,c
    call carryr64_1
; Old code:
;    movlw 0 ;round up the result
;    rlf r0,w ;set carry bit if req'd
;    call carryr64_2

;shift result right one byte
    movf r1,w
    movwf r0
    movf r2,w
    movwf r1
    movf r3,w
    movwf r2
    movf r4,w
    movwf r3
    movf r5,w
    movwf r4
    movf r6,w
    movwf r5
    clrf r6
mpyDone
    call Add64
    return



AccTimes10	;Multiply acc x 10
    ;copy acc to aux reg.
    call AccToR ;move acc contents to aux reg
    movlw 0x02
    call accLShiftN ;Shift left twice to get x4
    call Add64	; add the saved value back to the acc to get x5
    movlw 0x01
    call accLShiftN ;shift left once more to get x10
    return
AccLShiftN ;Shift acc left N times where N is in Wreg
    movwf xfrCount
LoopALSN
    bcf status,c
    rlf acc0,f
    rlf acc1,f
    rlf acc2,f
    rlf acc3,f
    rlf acc4,f
    rlf acc5,f
    rlf acc6,f
    rlf acc7,f
    decfsz xfrCount,f
    goto LoopALSN
    return
; ************************************************           *********
; ************************************************           *********
; ************************************************           *********
; ************************************************           *********
; ************************************************           *********
; ************************************************           *********
; ************************************************           *********
; ************************************************           *********
; ************************************************           *********
; ************************************************           *********
; ************************************************           *********

;
;  Utility subroutines
;
EEwriteNxt
    bank1
    incf eeadr,f
EEwrite ; Write to eeRam (data and address already set on entry)
    bank0
    bcf pir1,eeif
    bank1
;commented out following since interrupts not used
;    movf intcon,w
;    movwf saveIntcon  
    bsf eecon1,wren
    bcf intcon,gie    ; disable interrupts
    ;start of critical code
    movlw 0x55
    movwf eecon2
    movlw 0xAA
    movwf eecon2
    bsf eecon1,wr
    ;end of critical code
    bank0
    btfss pir1,eeif    ;wait for write to finish
    goto $-1
;commented out following since interrupts not used
;    movf saveIntcon,w
;    movwf Intcon    ;restore intcon register
    bank1
    movf eedata,w    ;compare data
    bsf eecon1,rd
    xorwf eedata,w
    return    ;if successful w=0 and z is set
getEEram
    bank1
    movwf eeadr
    bsf eecon1,rd
    movf eedata,w
    bank0 ;added bank0 2017-12-10
    return
getnxtEEram
    bank1
    incf eeadr,f
    bsf eecon1,rd
    movf eedata,w
    bank0 ;added bank0 2017-12-10
    return
;LCD routines
    ;Note that the LCD display is busy initializing for 10ms upon power up.
    ;So, the PIC program should not attempt any communications with the LCD
    ;until this time period elapses.
LCDinit
; Send the command to select 4-bit mode first
    movlw 0x02 ; Still in 8-bit mode, so command appears as 0x20 to LCD
    movwf LCDport
    call pulse_e
; Should now be in 4-bit mode - send init again
    movlw b'00101000' ;4 bit mode, 2 line display
    call Send_cmd
    movlw b'00001100' ;Display on, cursor off, blink off
    call Send_cmd
    movlw b'00000110' ;Left to right data entry
    goto Send_cmd
LCDclear
    bcf	LCDrs ; Command mode
    movlw b'00000001' ; clear display
    goto Send_cmd   ;A delay may be required after this command.
LCDhome
    movlw b'00000010' ; Return cursor to home
    call Send_cmd
    call delay1ms ;this command takes about 1.5ms to complete
    call delay1ms
    return
LCDline2
    movlw b'01000000' ; set address to 0x40 dec 64 (start posn of line 2)
LCDgoto
    iorlw b'10000000' ; target character position is in w reg
    goto Send_cmd
;set of routines to convert digits 0-9 to ascii and suppress leading 0's
LCDinitZerBlank
    clrf chrOffset
    bsf chrOffset,5 ;initialize the leading zero to blank
    return
DigitOutLast	;Call this to display last digit. This ensures that if value
		;is zero it will not be suppressed.
    bsf chrOffset,4 ;last digit so change space to ascii offset 30
DigitOut
    andlw 0x0F
    skpz
    bsf chrOffset,4 ;nonzero digit so change space to ascii offset 30
    addwf chrOffset,w
    call send_data
    return
;HexOut	;Send byte as 2 hex digits to display
;    movwf temp	;value is in w-reg
;    swapf temp,w
;    call HexDigits  ;lookup 1st ascii char
;    call send_data
;    movf temp,w
;    call HexDigits  ;lookup 2nd ascii char
;    call send_data
;    return
;Send text message stored in eeram
;   string is terminated with 0 byte (like a C string)
;   w-reg contains start position of test string
LCDeeTxt 
    bank1
    movwf eeadr
LCDeeTxt01
    bsf eecon1,rd
    movf eedata,w
    bank0
    skpnz ;check for termination byte
    return
    call Send_data
    bank1
    incf eeadr,f
    goto LCDeeTxt01
Send_cmd ;Sends a command to LCD, command byte is in w-reg
    clrf LCDmode
    goto Send_w
Send_data ;Sends data to LCD, data byte is in w-reg
    clrf LCDmode
    bsf LCDmode,4
send_w
    movwf LCDdatatemp
    swapf LCDdatatemp,w
    andlw 0x0F
    iorwf LCDmode,w ;set the RS bit
    movwf LCDport
    bsf	LCDenable
    bcf	LCDenable
    movf LCDdatatemp,w
    andlw 0x0F
    iorwf LCDmode,w ;set the RS bit
    movwf LCDport
    bsf	LCDenable
    bcf	LCDenable
    call delay40us
    return
pulse_e
    bsf	LCDenable
    bcf	LCDenable
    call delay40us
    return

;Misc 32/64 bit math routines
clrAcc64 ;clear 64 bit accumulator registers acc and r
    ClearNregisters d'16',acc0
    return

SaveAccHiInd ; Save high 4 bytes of acc to reg indirect
    movwf fsr ;start address is in w
    movf acc4,w
    movwf indf
    movf acc5,w
    incf fsr,f
    movwf indf
    movf acc6,w
    incf fsr,f
    movwf indf
    movf acc7,w
    incf fsr,f
    movwf indf
    return
SaveAccLoInd ; Save low 4 bytes of acc to reg indirect
    movwf fsr ;start address is in w
    movf acc0,w
    movwf indf
    movf acc1,w
    incf fsr,f
    movwf indf
    movf acc2,w
    incf fsr,f
    movwf indf
    movf acc3,w
    incf fsr,f
    movwf indf
    return
LoadAccHiInd ; Load high 4 bytes of acc from reg indirect
    movwf fsr ;start address is in w
    movf indf,w
    movwf acc4
    incf fsr,f
    movf indf,w
    movwf acc5
    incf fsr,f
    movf indf,w
    movwf acc6
    incf fsr,f
    movf indf,w
    movwf acc7
    return
LoadAccLoInd ; Load low 4 bytes of acc from reg indirect
    movwf fsr ;start address is in w
    movf indf,w
    movwf acc0
    incf fsr,f
    movf indf,w
    movwf acc1
    incf fsr,f
    movf indf,w
    movwf acc2
    incf fsr,f
    movf indf,w
    movwf acc3
    return
LoadRxLoInd ; Load low 4 bytes of aux reg r0 from reg indirect
    movwf fsr ;start address is in w
    movf indf,w
    movwf r0
    incf fsr,f
    movf indf,w
    movwf r1
    incf fsr,f
    movf indf,w
    movwf r2
    incf fsr,f
    movf indf,w
    movwf r3
    return
LoadRxHiInd ; Load low 4 bytes of aux reg r0 from reg indirect
    movwf fsr ;start address is in w
    movf indf,w
    movwf r4
    incf fsr,f
    movf indf,w
    movwf r5
    incf fsr,f
    movf indf,w
    movwf r6
    incf fsr,f
    movf indf,w
    movwf r7
    return
ClearNregistersInd
    ;Clear specified number of contiguous registers where number is in xfrCount
    ;and start register address is in w
    movwf fsr
LoopCNRI
    clrf indf
;2020-07-30: Added ,f to following instruction
    incf fsr,f
    decfsz xfrCount,f
    goto LoopCNRI
    return
ShiftUpNregistersInd
    ;Shift bytes up across N registers
    ;highest register is in w
    movwf fsr
LoopSUNRI
    decf fsr,f
    movf indf,w
    incf fsr,f
    movwf indf
    decf fsr,f
    decfsz xfrCount,f
    goto LoopSUNRI
    return
 

AccToR ;move acc contents to aux reg.
    ;No indirects here, so that calling code can use FSR without it
    ;getting corrupted here.
    movf acc0,w	
    movwf R0
    movf acc1,w
    movwf R1
    movf acc2,w
    movwf R2
    movf acc3,w
    movwf R3
    movf acc4,w
    movwf R4
    movf acc5,w
    movwf R5
    movf acc6,w
    movwf R6
    movf acc7,w
    movwf R7
    return

add32 ;32 bit add routine
    movf R0,w
    addwf acc0,f
    btfsc status,c
    call carry1
    movf R1,w
    addwf acc1,f
    btfsc status,c
    call carry2
    movf R2,w
    addwf acc2,f
    btfsc status,c
    incf acc3,f
    movf R3,w
    addwf acc3,f
    return
neg32	;Negate the accumulator
    comf acc0,f
    comf acc1,f
    comf acc2,f
    comf acc3,f
    incfsz acc0,f ;increment and check for carry
    return ;no carry, so return
carry1
    incfsz acc1,f
    return
carry2
    incfsz acc2,f
    return
    incf acc3,f
    return
add64 ;64 bit add routine
    movf R0,w
    addwf acc0,f
    btfsc status,c
    call carry64_1
    movf R1,w
    addwf acc1,f
    btfsc status,c
    call carry64_2
    movf R2,w
    addwf acc2,f
    btfsc status,c
    call carry64_3
add40 ;Entry point for 40 bit add
    movf R3,w
    addwf acc3,f
    btfsc status,c
    call carry64_4
    movf R4,w
    addwf acc4,f
    btfsc status,c
    call carry64_5
    movf R5,w
    addwf acc5,f
    btfsc status,c
    call carry64_6
    movf R6,w
    addwf acc6,f
    btfsc status,c
    incf acc7,f
    movf R7,w
    addwf acc7,f
    return
carry64_1   ;Chained skips to propagate carries
    incfsz acc1,f
    return
carry64_2
    incfsz acc2,f
    return
carry64_3
    incfsz acc3,f
    return
carry64_4
    incfsz acc4,f
    return
carry64_5
    incfsz acc5,f
    return
carry64_6
    incfsz acc6,f
    return
    incf acc7,f
    return
; Corresponding chained increments for aux R register carries
carryR64_1 
    incfsz r1,f
    return
carryr64_2
    incfsz r2,f
    return
carryr64_3
    incfsz r3,f
    return
carryr64_4
    incfsz r4,f
    return
carryr64_5
    incfsz r5,f
    return
carryr64_6
    incfsz r6,f
    return
    incf r7,f
    return

NegDecimal ; Negate 8-digit unpacked decimal number in acc0..7
    movlw 0x08
    movwf count
    movlw acc0
    movwf fsr
LoopND
    movf indf,w
    sublw 0x09
    movwf indf
    incf fsr,f
    decfsz count
    goto LoopND
    ;Acc now contains nines complement. Now increment to get tens complement
    incf acc0,f
    return
;otherwise continue with old code:
    ClearNregisters 8,r0
    incf r0,f
    ;continue to AddDecimal to 
AddDecimal ; Eight digit unpacked BCD add; operands in acc0..7 and r0..7
    call Add64 ; add the two 8-digit numbers
;Now adjust digits for carry
    movlw 0x08
    movwf xfrCount ;digit loop counter
    movlw acc0
    movwf fsr
    bcf status,c
DecAdjLoop
    btfsc status,c
    incf indf,f ; add carry from previous digit
    movlw 0xF6
    addwf indf,w
    btfsc status,c
    movwf indf ;store adjusted value back in register
noDecAdj
    incf fsr,f
    decfsz xfrCount,f
    goto DecAdjLoop
    return

; Delay routines
LongDelay
    movwf count
    movlw LongDelayMpy
LdelayLp
    call delay
    decfsz count,f
    goto LdelayLp
    return
delay40us   ;40 microsecond delay (approx), call is 2 cycles
    movlw Delay40usShort ;1 cycle
    movwf dlyctrL ;1 cycle
    decfsz dlyctrL,f ;1 cycle
    goto $-1 ;2 cycles
    return ;2 cycles

Delay1mS	 ; 00002 cycles for subroutine call
    movlw Delay1msLong      ; 00001 cycle - load long delay count
    call delay	 ; 770*w+4 cycles
    movlw Delay1msShort    ; 00001 cycle - load short delay count
    movwf dlyctrL ; 00001 cycle
    decfsz dlyctrL,f ; 3cycles/iteration
                 ; = 00216 total cycles (00744 for 4MHz)
    goto $-1	 ; -0001 cycle for loop exit
    nop		 ; 00001 cycle short tweak
    Return	 ; 00002 cycles, total 997 cycles

Delay10mS    ; 00002 cycles for subroutine call
    movlw Delay10msLong    ; 00001 cycle - long delay count
    call delay    ; 770*w+4 cycles
    movlw Delay10msShort    ; 00001 cycle - short delay count
    movwf dlyctrL    ; 00001 cycle
    decfsz dlyctrL,f    ; 3cycles/iteration
                        ; = 00462 total cycles (00744 for 4MHz)
    goto $-1    ; -0001 cycle for loop exit
    goto $+1    ; 00002 cycles - a short tweak
     goto $+1    ; +2 cycles reqd for 3.579 MHz xtal only
;   nop         ; +1 cycle reqd for 4 MHz xtal only
    Return    ; 00002 cycles, total 9997 cycles
delay ; Variable Delay subroutine: delay cycles = 770*w+4
    ; 00002 cyc for entry call
    movwf    dlyctrH    ; 00001 cyc, w contains delay value
subloop
    decfsz    dlyctrL,f    ; 00768 cyc for loop * w
    goto $-1    ; -0001 cyc for loop exit * w
    decfsz dlyctrH,f    ; 00001 * w
    goto subloop    ; 00002 * w
    ; -0001 for loop exit
    return ; 00002
;
; Location of stored factory configuration bits
;
	org 0x3ff
OscConst
	retlw	0x98
;Start of eeram
    org eebase
#ifdef UseSeparators
msg01
    dt "F"
    de 0
msg02
    dt "+"
    de 0
#else
msg01
    dt "Fo:"
    de 0
msg02
    dt "F+:"
    de 0
#endif
msgRightArrow
    de 0x7E
    de 0
msgLeftArrow
    de 0x7F
    de 0

ftw
    ; Saved value of FTW base value. Default is 125 for AD9850.
    de 0xAD,0x4D,0xD0,0x17,0x5C,0x22 ;weight per digit coefficient 125Mhz
eeCtlWd
    de 0x00 ;control word for AD9850
;The following allows provision to modify the saved ftw coefficient, but have the 
;ability to restore the original setting if things go horribly wrong.

ftwFactorySetting1
    ; RefOsc180MHz
    de 0x9C,0xE0,0xDE,0x65,0xDC,0x17 ;weight per digit coefficient 180Mhz
    de 0x01 ;control word for AD9851
msg180
    dt "180 MHz ref"
    de 0
ftwFactorySetting2
    ; RefOsc125MHz
    de 0xAD,0x4D,0xD0,0x17,0x5C,0x22 ;weight per digit coefficient 125Mhz
    de 0x00 ;control word for AD9850
msg125
    dt "125 MHz ref"
    de 0
    end